# Generated by Django 4.2.15 on 2024-08-30 09:00

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_gar", "0005_garinstitution_project_code"),
    ]

    operations = [
        migrations.AddField(
            model_name="garinstitution",
            name="id_ent",
            field=models.CharField(
                max_length=255, null=True, verbose_name="ID de l'ent"
            ),
        ),
    ]
