{% extends "admin/change_list.html" %}
{% load i18n %}
{% block object-tools-items %}
    <li>
        <a href="{% url 'admin:django_gar_garinstitution_generate_allocations_report' %}">
            Rapport d’affectations
        </a>
    </li>
    {{ block.super }}
{% endblock %}
